#include "IZ_wsclient.h"

IZ_ProcedureResult IZ_WSClientCallback(
	struct lws* wsi,
	enum lws_callback_reasons reason,
	void* user,
	void* in,
	size_t len
) {
	switch (reason) {
		case LWS_CALLBACK_CLIENT_CONNECTION_ERROR:
			printf("Client Connection Error %llu\n", len);
			return 1;
		case LWS_CALLBACK_CLIENT_CLOSED:
			printf("Client Closed %llu\n", len);
			return 0;
		case LWS_CALLBACK_CLIENT_RECEIVE:
			printf("Client Receive %llu\n", len);
			break;
		case LWS_CALLBACK_CLIENT_ESTABLISHED:
			printf("Client Established %llu\n", len);
			break;
		default:
			break;
	}

	return lws_callback_http_dummy(wsi, reason, user, in, len);
}

IZ_ProcedureResult IZ_WSClientCreateConnection(IZ_WSClient* client, IZ_WSClientConnectParams params) {
	static struct lws_client_connect_info info;
	memset(&info, 0, sizeof info);

	info.context = client->context;
	info.port = params.port;
	info.address = params.address;
	info.host = params.address;
	info.origin = params.address;
	info.protocol = params.protocol;
	info.path = params.path;
	info.userdata = params.userdata;
//	info.ssl_connection = (
//		LCCSCF_ALLOW_SELFSIGNED
//		| LCCSCF_ALLOW_INSECURE
//		| LCCSCF_SKIP_SERVER_CERT_HOSTNAME_CHECK
//		| LCCSCF_ALLOW_EXPIRED
//	);

	client->connection = lws_client_connect_via_info(&info);
	if (!client->connection) {
		return 1;
	}

	return 0;
}

IZ_ProcedureResult IZ_WSClientCreateContext(IZ_WSClient* client, void* userdata) {
	static struct lws_protocols protocols[] = {
		{
			.name = "http",
			.callback = IZ_WSClientCallback,
			.per_session_data_size = 0,
			.rx_buffer_size = 0,
			.id = 0,
			.user = NULL,
			.tx_packet_size = 0,
		},
		LWS_PROTOCOL_LIST_TERM,
	};
	protocols[0].user = userdata;
	static struct lws_context_creation_info info;
	memset(&info, 0, sizeof info);
	info.port = CONTEXT_PORT_NO_LISTEN,
	info.protocols = protocols,
	info.options = LWS_SERVER_OPTION_DO_SSL_GLOBAL_INIT,

		client->context = lws_create_context(&info);
	if (!client->context) {
		return 1;
	}

	return 0;
}

void IZ_WSClientInitialize(IZ_WSClient* client) {
	// TODO delegate logs to SDL
	//lws_set_log_level(0, NULL);
	client->context = NULL;
	client->connection = NULL;
}

IZ_ProcedureResult IZ_WSClientConnect(IZ_WSClient* client, IZ_WSClientConnectParams params) {
	if (IZ_WSClientCreateContext(client, params.userdata)) {
		return 1;
	}

	if (IZ_WSClientCreateConnection(client, params)) {
		return 2;
	}

	return 0;
}

IZ_ProcedureResult IZ_WSClientHandle(IZ_WSClient* client) {
	i32 response = lws_service(client->context, 0);
	if (response < 0) {
		return 1;
	}

	return 0;
}

void IZ_WSClientDisconnect(IZ_WSClient* client) {
	lws_context_destroy(client->context);
}
